import { h } from "@stencil/core";
import * as mr from "@pie-lib/math-rendering";
import { PieLoader, DEFAULT_ENDPOINTS } from "../../pie-loader";
import { pieContentFromConfig } from "../../utils/utils";
import parseNpm from "parse-package-name";
import _isEqual from "lodash/isEqual";
import _isEmpty from "lodash/isEmpty";
import { addComplexRubric, addMultiTraitRubric, addPackageToContent, addRubric } from "../../rubric-utils";
import { ModelUpdatedEvent, InsertImageEvent, DeleteImageEvent, InsertSoundEvent, DeleteSoundEvent } from "@pie-framework/pie-configure-events";
import { DataURLImageSupport } from "./dataurl-image-support";
import { DataURLUploadSoundSupport } from "./dataurl-upload-sound-support";
import { VERSION } from "../../version";
import cloneDeep from "lodash/cloneDeep";
/**
 * Pie Author will load a Pie Content model for authoring.
 * It needs to be run in the context
 */
export class Author {
    constructor() {
        this._modelLoadedState = false;
        /**
         * Allows disabling of the default behaviour which is to look up and load the JS bundle that define the Custom Elements
         * used by the item config.
         * This if for advanced use cases when using the pie-player in a container that is managing loading of Custom Elements and Controllers.
         */
        this.disableBundler = false;
        /**
         * Adds a preview view which will render the content in another tab as it may appear to a student or instructor.
         */
        this.addPreview = false;
        this.elementsLoaded = false;
        this.pieLoader = new PieLoader();
        this.fileInput = null;
        this.imageHandler = null;
        /** external providers can set this if they need to upload the assets to the cloud etc. by default we use data urls */
        this.imageSupport = new DataURLImageSupport();
        /** external providers can set this if they need to upload the assets to the cloud etc. by default we use data urls */
        this.uploadSoundSupport = new DataURLUploadSoundSupport();
        this.version = VERSION;
        this.isAdvancedItemConfig = (config) => config.pie;
        this.checkComplexRubric = config => {
            const shouldHaveComplexRubric = config.models.filter(model => model.rubricEnabled).length;
            const hasComplexRubric = Object.keys(config.elements).filter(key => config.elements[key].indexOf('complex-rubric') >= 0).length;
            return {
                shouldHaveComplexRubric,
                hasComplexRubric
            };
        };
        this.handleFileInputChange = (e) => {
            const input = e.target;
            if (!this.imageHandler) {
                console.error("no image handler - but file input change triggered?");
                return;
            }
            const files = input.files;
            if (files.length < 1 || !files[0]) {
                this.imageHandler.cancel();
                this.imageHandler = null;
            }
            else {
                const file = files[0];
                this.imageHandler.fileChosen(file);
                this.fileInput.value = "";
                this.imageSupport.insert(file, (e, src) => {
                    if (e) {
                        console.warn("error inserting image: ", e.message);
                        console.error(e);
                    }
                    this.imageHandler.done(e, src);
                    this.imageHandler = null;
                }, (percent, bytes, total) => this.imageHandler.progress(percent, bytes, total));
            }
        };
        this.handleInsertImage = (e) => {
            console.log("[handleInsertImage]", e);
            this.imageHandler = e.detail;
            if (!e.detail.isPasted) {
                this.fileInput.click();
            }
        };
        this.handleDeleteImage = (e) => {
            console.log("[handleDeleteImage ..]", e);
            this.imageSupport.delete(e.detail.src, e.detail.done);
        };
        this.handleInsertSound = (e) => {
            console.log("[handleInsertSound]", e);
            this.uploadSoundSupport.insert(e.detail.fileChosen, e.detail.done);
        };
        this.handleDeleteSound = (e) => {
            console.log("[handleDeleteSound ..]", e);
            this.uploadSoundSupport.delete(e.detail.src, e.detail.done);
        };
    }
    async validateModels() {
        if (!this.pieContentModel || !this.pieContentModel.models) {
            console.error('No pie content model');
            return { hasErrors: false, validatedModels: {} };
        }
        return (this.pieContentModel.models || []).reduce((acc, model) => {
            let pieEl = this.el.querySelector(`[id='${model.id}']`);
            !pieEl && (pieEl = this.el.querySelector(`[pie-id='${model.id}']`));
            if (pieEl) {
                const pieElName = pieEl.tagName.toLowerCase().split("-config")[0];
                const controller = this.pieLoader.getController(pieElName);
                const packageName = parseNpm(this.pieContentModel.elements[pieElName]).name;
                const configuration = this.configSettings && this.configSettings[packageName] || {};
                if (controller && controller.validate) {
                    // here we call controller.validate which returns an object with all the errors
                    const errors = controller.validate(model, configuration);
                    // here we can update the model in author, so we can set errors
                    pieEl.model = Object.assign(Object.assign({}, model), { errors });
                    // for ebsr
                    if (errors && errors.partA && errors.partB) {
                        acc.hasErrors = acc.hasErrors || (!_isEmpty(errors.partA) || !_isEmpty(errors.partB));
                    }
                    else {
                        // here we return a boolean value if models are valid or not
                        acc.hasErrors = acc.hasErrors || !_isEmpty(errors);
                    }
                    acc.validatedModels = Object.assign(Object.assign({}, acc.validatedModels), { [model.id]: Object.assign(Object.assign({}, model), { errors }) });
                }
                else {
                    acc.validatedModels = Object.assign(Object.assign({}, acc.validatedModels), { [model.id]: model });
                }
            }
            return acc;
        }, { hasErrors: false, validatedModels: {} });
    }
    getRenderMarkup() {
        let markup = this.pieContentModel ? this.pieContentModel.markup : "";
        if (markup) {
            Object.keys(this.pieContentModel.elements).forEach(key => {
                markup = markup.split(key).join(key + "-config");
            });
            return markup;
        }
    }
    async watchConfig(newValue, oldValue) {
        if (newValue && !_isEqual(newValue, oldValue)) {
            try {
                this.elementsLoaded = false;
                this._modelLoadedState = false;
                this.pieContentModel = pieContentFromConfig(newValue);
                this.addConfigTags(this.pieContentModel);
                this.loadPieElements();
                // if it should have complex-rubric, reset config
                const { shouldHaveComplexRubric, hasComplexRubric } = this.checkComplexRubric(this.pieContentModel);
                if (shouldHaveComplexRubric && !hasComplexRubric) {
                    const newConfig = await this.addComplexRubric();
                    if (this.isAdvancedItemConfig(this.config)) {
                        this.config = Object.assign(Object.assign({}, this.config), { pie: newConfig });
                    }
                    else {
                        this.config = newConfig;
                    }
                }
            }
            catch (error) {
                console.log(`ERROR ${error}`);
            }
        }
    }
    addConfigTags(c) {
        if (!c.markup && c.models) {
            const tags = c.models.map(model => {
                return `<${model.element}-config id="${model.id}"></${model.element}-config>`;
            });
            c.markup = tags.join("");
        }
    }
    async watchElementsLoaded(newValue, oldValue) {
        if (newValue && !oldValue) {
            await this.updateModels();
        }
    }
    async updateModels() {
        if (this.pieContentModel &&
            this.pieContentModel.elements &&
            this.pieContentModel.markup) {
            if (!this.pieContentModel.models) {
                this.pieContentModel.models = [];
            }
            const tempDiv = this.doc.createElement("div");
            tempDiv.innerHTML = this.pieContentModel.markup;
            const elsWithId = tempDiv.querySelectorAll("[id]");
            // set up a model for each pie defined in the markup
            elsWithId.forEach(el => {
                const pieElName = el.tagName.toLowerCase().split("-config")[0];
                // initialize emtpy model if this is a pie
                if (this.pieContentModel.elements[pieElName]) {
                    const elementId = el.getAttribute("id");
                    if (!this.pieContentModel.models.find(m => m.id === elementId)) {
                        const model = { id: elementId, element: pieElName };
                        this.pieContentModel.models.push(model);
                    }
                }
            });
            tempDiv.remove();
        }
        if (this.pieContentModel && this.pieContentModel.models) {
            this.pieContentModel.models.map(model => {
                let pieEl = this.el.querySelector(`[id='${model.id}']`);
                !pieEl && (pieEl = this.el.querySelector(`[pie-id='${model.id}']`));
                if (pieEl) {
                    const pieElName = pieEl.tagName.toLowerCase().split("-config")[0];
                    const packageName = parseNpm(this.pieContentModel.elements[pieElName]).name;
                    pieEl.model = model;
                    if (this.configSettings && this.configSettings[packageName]) {
                        pieEl.configuration = this.configSettings[packageName];
                    }
                }
            });
            if (this._modelLoadedState === false) {
                this._modelLoadedState = true;
                this.modelLoaded.emit(this.pieContentModel);
            }
        }
    }
    componentDidUnload() {
        this.el.removeEventListener(InsertImageEvent.TYPE, this.handleInsertImage);
        this.el.removeEventListener(DeleteImageEvent.TYPE, this.handleDeleteImage);
        this.el.removeEventListener(InsertSoundEvent.TYPE, this.handleInsertSound);
        this.el.removeEventListener(DeleteSoundEvent.TYPE, this.handleDeleteSound);
        if (this.fileInput) {
            this.fileInput.removeEventListener("change", this.handleFileInputChange);
        }
    }
    async componentWillLoad() {
        if (this.config) {
            this.watchConfig(this.config, {});
        }
        // Note: cannot use the @Listen decorator as creates bundling problems due
        // to `.` in event name.
        this.el.addEventListener(ModelUpdatedEvent.TYPE, async (e) => {
            // set the internal model
            // emit a content-item level event with the model
            if (this.pieContentModel && e.update) {
                this.pieContentModel.models.forEach(m => {
                    if (m.id === e.update.id && m.element === e.update.element) {
                        Object.assign(m, e.update);
                    }
                });
            }
            if (this._modelLoadedState) {
                this.modelUpdated.emit(this.pieContentModel);
            }
            const { shouldHaveComplexRubric, hasComplexRubric } = this.checkComplexRubric(this.pieContentModel);
            if (shouldHaveComplexRubric && !hasComplexRubric) {
                const newConfig = await this.addComplexRubric();
                if (this.isAdvancedItemConfig(this.config)) {
                    this.config = Object.assign(Object.assign({}, this.config), { pie: newConfig });
                }
                else {
                    this.config = newConfig;
                }
            }
            if (!shouldHaveComplexRubric && hasComplexRubric) {
                const rubricElements = Object.keys(this.pieContentModel.elements).filter(key => this.pieContentModel.elements[key].indexOf('complex-rubric') >= 0);
                const newConfig = this.removeRubricItemTypes(rubricElements);
                if (this.isAdvancedItemConfig(this.config)) {
                    this.config = Object.assign(Object.assign({}, this.config), { pie: newConfig });
                }
                else {
                    this.config = newConfig;
                }
            }
        });
        this.el.addEventListener(InsertImageEvent.TYPE, this.handleInsertImage);
        this.el.addEventListener(DeleteImageEvent.TYPE, this.handleDeleteImage);
        this.el.addEventListener(InsertSoundEvent.TYPE, this.handleInsertSound);
        this.el.addEventListener(DeleteSoundEvent.TYPE, this.handleDeleteSound);
    }
    async componentDidLoad() {
        await this.afterRender();
    }
    async componentDidUpdate() {
        await this.afterRender();
        if (this.fileInput) {
            this.fileInput.addEventListener("change", this.handleFileInputChange);
        }
    }
    async loadPieElements() {
        if (this.config && !this.disableBundler) {
            let endpoints = DEFAULT_ENDPOINTS.prod;
            if (this.bundleHost &&
                ["dev", "stage", "prod"].includes(this.bundleHost)) {
                endpoints = DEFAULT_ENDPOINTS[this.bundleHost];
            }
            if (this.bundleEndpoints) {
                endpoints = this.bundleEndpoints;
            }
            await this.pieLoader.loadCloudPies({
                content: this.pieContentModel,
                doc: this.doc,
                endpoints,
                useCdn: false
            });
        }
    }
    renderMath() {
        setTimeout(() => {
            mr.renderMath(this.el);
        }, 50);
    }
    async afterRender() {
        if (this.pieContentModel &&
            this.pieContentModel.markup &&
            !this.elementsLoaded) {
            const elements = Object.keys(this.pieContentModel.elements).map(el => ({
                name: el,
                tag: `${el}-config`
            }));
            const loadedInfo = await this.pieLoader.elementsHaveLoaded(elements);
            if (loadedInfo.val &&
                !!loadedInfo.elements.find(el => this.pieContentModel.elements[el.name])) {
                this.elementsLoaded = true;
                this.renderMath();
            }
        }
    }
    removeRubricFromMarkup(rubricElements) {
        const tempDiv = this.doc.createElement("div");
        tempDiv.innerHTML = this.pieContentModel.markup;
        const elsWithId = tempDiv.querySelectorAll("[id]");
        elsWithId.forEach(el => {
            const pieElName = el.tagName.toLowerCase().split("-config")[0];
            if (rubricElements.includes(pieElName)) {
                try {
                    tempDiv.querySelector(`#${el.id}`).remove();
                }
                catch (e) {
                    console.log(e.toString());
                }
            }
        });
        const newMarkup = tempDiv.innerHTML;
        tempDiv.remove();
        return newMarkup;
    }
    removeRubricItemTypes(rubricElements) {
        if (!rubricElements.length || !this.pieContentModel.models) {
            return this.pieContentModel;
        }
        const pieContentModel = cloneDeep(this.pieContentModel);
        // delete the rubric and multi-trait-rubric elements
        rubricElements.forEach(rubricElementKey => delete pieContentModel.elements[rubricElementKey]);
        // delete the rubric and multi-trait-rubric models
        pieContentModel.models = pieContentModel.models.filter(model => !rubricElements.includes(model.element));
        // delete the rubric and multi-trait-rubric nodes from markup
        pieContentModel.markup = this.removeRubricFromMarkup(rubricElements);
        return pieContentModel;
    }
    async addComplexRubric() {
        const complexRubricModel = {
            id: "complex-rubric",
            element: "pie-complex-rubric",
        };
        // add complex-rubric
        addPackageToContent(this.pieContentModel, "@pie-element/complex-rubric", complexRubricModel);
        return addComplexRubric(this.pieContentModel);
    }
    /**
     * Utility method to add a `@pie-element/rubric` section to an item config when creating an item should be used before setting the config.
     *
     * @deprecated this method is for temporary use, will be removed at next major release
     *
     * @param config the item config to mutate
     * @param rubricModel
     */
    async addRubricToConfig(config, rubricModel) {
        if (!rubricModel) {
            rubricModel = {
                id: "rubric",
                element: "pie-rubric",
                points: ["", "", "", ""],
                maxPoints: 4,
                excludeZero: false
            };
        }
        const configPieContent = pieContentFromConfig(config);
        addPackageToContent(configPieContent, "@pie-element/rubric", rubricModel);
        return addRubric(configPieContent);
    }
    /**
     * Utility method to add a `@pie-element/multi-trait-rubric` section to an item config when creating an item should be used before setting the config.
     **
     * @param config the item config to mutate
     * @param multiTraitRubricModel
     */
    async addMultiTraitRubricToConfig(config, multiTraitRubricModel) {
        if (!multiTraitRubricModel) {
            multiTraitRubricModel = {
                id: "multi-trait-rubric",
                element: "pie-multi-trait-rubric",
                visibleToStudent: true,
                halfScoring: false,
                excludeZero: true,
                pointLabels: true,
                description: false,
                standards: false,
                scales: [
                    {
                        maxPoints: 4,
                        scorePointsLabels: ['', '', '', ''],
                        traitLabel: 'Trait',
                        traits: [
                            {
                                name: '',
                                standards: [],
                                description: '',
                                scorePointsDescriptors: [
                                    '',
                                    '',
                                    '',
                                    '',
                                    '',
                                ],
                            },
                        ]
                    }
                ]
            };
        }
        const configPieContent = pieContentFromConfig(config);
        addPackageToContent(configPieContent, "@pie-element/multi-trait-rubric", multiTraitRubricModel);
        return addMultiTraitRubric(configPieContent);
    }
    render() {
        if (this.pieContentModel && this.pieContentModel.markup) {
            const markup = this.getRenderMarkup();
            if (this.addPreview) {
                return (h("pie-preview-layout", { config: this.config },
                    h("div", { slot: "configure" },
                        h("pie-spinner", { active: !this.elementsLoaded },
                            h("div", { innerHTML: markup }))),
                    h("input", { type: "file", hidden: true, ref: r => (this.fileInput = r), style: { display: 'none' } })));
            }
            else {
                return (h("pie-spinner", { active: !this.elementsLoaded },
                    h("div", { innerHTML: markup }),
                    h("input", { type: "file", hidden: true, ref: r => (this.fileInput = r), style: { display: 'none' } })));
            }
        }
        else {
            return h("pie-spinner", null);
        }
    }
    static get is() { return "pie-author"; }
    static get originalStyleUrls() { return {
        "$": ["../components.css"]
    }; }
    static get styleUrls() { return {
        "$": ["../components.css"]
    }; }
    static get properties() { return {
        "bundleHost": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "Optionally specifies the back-end that builds and hosts javascript bundles for rendering assessment items.\nThis property lets you choose which environment to use, from 'dev' , 'stage' or 'prod' environments.\nUntil 1.0 will default to 'stage'."
            },
            "attribute": "bundle-host",
            "reflect": false
        },
        "bundleEndpoints": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "BundleEndpoints",
                "resolved": "BundleEndpoints",
                "references": {
                    "BundleEndpoints": {
                        "location": "import",
                        "path": "../../pie-loader"
                    }
                }
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "Provide this property override the default endpoints used by the player to retrieve JS\nbundles. Must be set before setting the config property.\nMost users will not need to use this property."
            }
        },
        "disableBundler": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Allows disabling of the default behaviour which is to look up and load the JS bundle that define the Custom Elements\nused by the item config.\nThis if for advanced use cases when using the pie-player in a container that is managing loading of Custom Elements and Controllers."
            },
            "attribute": "disable-bundler",
            "reflect": false,
            "defaultValue": "false"
        },
        "addRubric": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "If set the player will add a rubric authoring interaction to the config"
            },
            "attribute": "add-rubric",
            "reflect": false
        },
        "addPreview": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Adds a preview view which will render the content in another tab as it may appear to a student or instructor."
            },
            "attribute": "add-preview",
            "reflect": false,
            "defaultValue": "false"
        },
        "config": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "ItemConfig",
                "resolved": "AdvancedItemConfig | PieContent",
                "references": {
                    "ItemConfig": {
                        "location": "import",
                        "path": "../../interface"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The Pie config model."
            }
        },
        "configSettings": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "{ [packageName: string]: Object }",
                "resolved": "{ [packageName: string]: Object; }",
                "references": {
                    "Object": {
                        "location": "global"
                    }
                }
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "To customize the standard behaviour provided by interaction configuration views you can\nprovide settings key-ed by the package name.  e.g.\n\n`{ '@pie-element/inline-choice': { promptLabel: 'Item Stem' } }`\n\nThe settings that are configurable for each authoring view are documented in\nthe `@package-name/docs` folder for each package."
            }
        },
        "imageSupport": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "ExternalImageSupport",
                "resolved": "ExternalImageSupport",
                "references": {
                    "ExternalImageSupport": {
                        "location": "import",
                        "path": "./dataurl-image-support"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "external providers can set this if they need to upload the assets to the cloud etc. by default we use data urls"
            },
            "defaultValue": "new DataURLImageSupport()"
        },
        "uploadSoundSupport": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "ExternalUploadSoundSupport",
                "resolved": "ExternalUploadSoundSupport",
                "references": {
                    "ExternalUploadSoundSupport": {
                        "location": "import",
                        "path": "./dataurl-upload-sound-support"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "external providers can set this if they need to upload the assets to the cloud etc. by default we use data urls"
            },
            "defaultValue": "new DataURLUploadSoundSupport()"
        },
        "version": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "version",
            "reflect": false,
            "defaultValue": "VERSION"
        }
    }; }
    static get contextProps() { return [{
            "name": "doc",
            "context": "document"
        }]; }
    static get states() { return {
        "elementsLoaded": {},
        "fileInput": {}
    }; }
    static get events() { return [{
            "method": "modelUpdated",
            "name": "modelUpdated",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "Emmitted when the model for the content has been updated within the ui due to user action."
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }, {
            "method": "modelLoaded",
            "name": "modelLoaded",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "Emmitted when the content models in the config have been set on the content"
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }]; }
    static get methods() { return {
        "validateModels": {
            "complexType": {
                "signature": "() => Promise<any>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    },
                    "PieElement": {
                        "location": "import",
                        "path": "../../interface"
                    },
                    "PieController": {
                        "location": "import",
                        "path": "../../interface"
                    }
                },
                "return": "Promise<any>"
            },
            "docs": {
                "text": "",
                "tags": []
            }
        },
        "addRubricToConfig": {
            "complexType": {
                "signature": "(config: ItemConfig, rubricModel?: any) => Promise<PieContent>",
                "parameters": [{
                        "tags": [{
                                "text": "config the item config to mutate",
                                "name": "param"
                            }],
                        "text": "the item config to mutate"
                    }, {
                        "tags": [{
                                "text": "rubricModel",
                                "name": "param"
                            }],
                        "text": ""
                    }],
                "references": {
                    "Promise": {
                        "location": "global"
                    },
                    "PieContent": {
                        "location": "import",
                        "path": "../../interface"
                    },
                    "ItemConfig": {
                        "location": "import",
                        "path": "../../interface"
                    },
                    "PieModel": {
                        "location": "import",
                        "path": "../../interface"
                    }
                },
                "return": "Promise<PieContent>"
            },
            "docs": {
                "text": "Utility method to add a `@pie-element/rubric` section to an item config when creating an item should be used before setting the config.",
                "tags": [{
                        "name": "deprecated",
                        "text": "this method is for temporary use, will be removed at next major release"
                    }, {
                        "name": "param",
                        "text": "config the item config to mutate"
                    }, {
                        "name": "param",
                        "text": "rubricModel"
                    }]
            }
        },
        "addMultiTraitRubricToConfig": {
            "complexType": {
                "signature": "(config: ItemConfig, multiTraitRubricModel?: any) => Promise<PieContent>",
                "parameters": [{
                        "tags": [{
                                "text": "config the item config to mutate",
                                "name": "param"
                            }],
                        "text": "the item config to mutate"
                    }, {
                        "tags": [{
                                "text": "multiTraitRubricModel",
                                "name": "param"
                            }],
                        "text": ""
                    }],
                "references": {
                    "Promise": {
                        "location": "global"
                    },
                    "PieContent": {
                        "location": "import",
                        "path": "../../interface"
                    },
                    "ItemConfig": {
                        "location": "import",
                        "path": "../../interface"
                    },
                    "PieModel": {
                        "location": "import",
                        "path": "../../interface"
                    }
                },
                "return": "Promise<PieContent>"
            },
            "docs": {
                "text": "Utility method to add a `@pie-element/multi-trait-rubric` section to an item config when creating an item should be used before setting the config.\n*",
                "tags": [{
                        "name": "param",
                        "text": "config the item config to mutate"
                    }, {
                        "name": "param",
                        "text": "multiTraitRubricModel"
                    }]
            }
        }
    }; }
    static get elementRef() { return "el"; }
    static get watchers() { return [{
            "propName": "config",
            "methodName": "watchConfig"
        }, {
            "propName": "elementsLoaded",
            "methodName": "watchElementsLoaded"
        }]; }
}
